/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package membrane.glue;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.juliak.membrane.CustomFactory;
import org.objectweb.fractal.juliak.membrane.MembraneDef;
import org.objectweb.fractal.juliak.membrane.Membranes;
import org.objectweb.fractal.util.Fractal;

/**
 * This class illustrates the definition of new control membranes.
 * 
 * This example defines a control membrane which extends the control membrane of
 * primitive components with a logger controller (as defined by the
 * <a href="http://dream.objectweb.org">Dream</a<
 * framework). This new membrane, represented as a control {@link Component}, is
 * used to instantiate an application level component.
 * 
 * Two techniques are illustrated. The first technique consists in instanciating
 * the control membrane with the Fractal API (just like any Fractal component)
 * and then using this component as the control part for instanciating
 * another component. The second techniques consists in registering the new
 * control membrane with the membrane repository.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class Main {
    
    public static void main(String[] args) throws Exception {
        
        Component boot = Fractal.getBootstrapComponent();
        TypeFactory tf = Fractal.getTypeFactory(boot);
        GenericFactory gf = Fractal.getGenericFactory(boot);
        
        /*
         * Create the component type.
         */
        ComponentType type =
            tf.createFcType(
                new InterfaceType[]{
                    tf.createFcItfType(
                        "r", Runnable.class.getName(), false, false, false)
                }
            );
        
        /*
         * Use a component as the control membrane for creating another
         * component.
         */
        String adl = LoggablePrimitive.class.getName();
        Class cl = Class.forName(adl);
        CustomFactory cf = (CustomFactory) cl.newInstance();
        cf.setFcGenericFactory(gf);
        cf.setFcTypeFactory(tf);
        Component membrane = cf.newFcInstance();
        Component c1 = gf.newFcInstance(type, membrane, C.class.getName());
        run(c1);
        
        /*
         * Register a new membrane with the membrane repository.
         */
        MembraneDef mdef = new MembraneDef("loggablePrimitive",adl);
        Membranes membranes = Membranes.get();
        membranes.register(mdef);
        Component c2 = gf.newFcInstance(type, "loggablePrimitive", C.class.getName());
        run(c2);
    }
    
    private static void run( Component c ) throws Exception {
        /*
         * Register the component with the logger controller.
         */
        LoggerControllerRegister lcr = (LoggerControllerRegister)
            c.getFcInterface("/logger-controller-register");
        Loggable content = (Loggable) c.getFcInterface("/content");
        lcr.register("myLogger", content);
        
        /*
         * Start and run the application.
         */
        Fractal.getLifeCycleController(c).startFc();
        ((Runnable)c.getFcInterface("r")).run();   
    }
}
