/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.asm;

import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.loader.Loader;
import org.objectweb.fractal.juliak.Juliak;
import org.objectweb.fractal.juliak.factory.ClassDefinition;
import org.objectweb.fractal.juliak.factory.ClasseDefinitions;
import org.objectweb.fractal.juliak.platform.Platform;

/**
 * Root class for {@link InterfaceType}-based class generators.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public abstract class AbstractClassGenerator implements ClassGenerator {
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    abstract protected String getClassNameSuffix( InterfaceType it );
    
    /**
     * Return true if the mode (IN, OUT or IN_OUT) of the class generator matchs
     * the mode (client or server) of the given interface type.
     */
    abstract public boolean match( InterfaceType it );

    /**
     * Generate the bytecode of a class implementing the given Fractal interface
     * type.
     * 
     * @param targetClassname    the name of the class to generate
     * @param it  the corresponding interface type
     * @param ic  the context for initializing the generated class
     * @return    the bytecode of the generated class
     */
    abstract public byte[] generate(
            String targetClassname, InterfaceType it,
            InitializationContext ic )
    throws InstantiationException;

    /**
     * Generate a class implementing the given Fractal interface type.
     * 
     * @param it  the interface type
     * @param ic  the context for initializing the generated class
     * @return    a class implementing the given Fractal interface type
     */
    public Class generate( InterfaceType it, InitializationContext ic )
    throws InstantiationException {
        
        /*
         * Check whether the class has already been requested.
         */
        ClasseDefinitions clrep = ClasseDefinitions.get();
        String targetClassname = getTargetClassName(it);
        if( clrep.containsClass(targetClassname) ) {
            ClassDefinition cd = clrep.getClass(targetClassname);
            return cd.cl;
        }
        
        /*
         * Try to load the class from the classpath.
         */
        try {
            return Platform.get().loadClass(targetClassname);
        }
        catch( ClassNotFoundException cnfe ) {}
        
        /*
         * Generate the class and register it.
         */
        byte[] bytecode = generate(targetClassname,it,ic);
        ClassDefinition cd = clrep.registerClass(targetClassname,bytecode);
        
        return cd.cl;
    }

    /**
     * Return the name of the class generated by the current class generator
     * for the specified interface type.
     */
    public String getTargetClassName( InterfaceType it ) {
        final String javaItfSignature = it.getFcItfSignature();
        final String name = AbstractClassGenerator.getSimpleName(javaItfSignature);
        final String pkgName = AbstractClassGenerator.getPkgName(javaItfSignature);
        final String suffix = getClassNameSuffix(it);
        final String targetClassname =
            AbstractClassGenerator.getGenPkgName(pkgName) + "." + name + suffix;
        return targetClassname;
    }
    
    /**
     * Given a fully qualified name, return the corresponding simple name, i.e.
     * strip the package name.
     */
    private static String getSimpleName( String fqn ) {
        int lastdot = fqn.lastIndexOf('.');
        String name = (lastdot==-1) ? fqn : fqn.substring(lastdot+1);
        return name;
    }

    /**
     * Given a fully qualified name, return the corresponding package name.
     */
    private static String getPkgName( String fqn ) {
        int lastdot = fqn.lastIndexOf('.');
        String name = (lastdot==-1) ? "" : fqn.substring(0,lastdot);
        return name;
    }

    /**
     * Given a source package name, return the name of the destination package
     * which will contain the code generated for the classes of the source
     * package.
     */
    private static String getGenPkgName( String pkgName ) {        
        return
            (pkgName.length()==0) ? 
            genByJuliakPkgName : genByJuliakPkgName + "." + pkgName;
    }

    /** The package name extension for classes generated by Juliak. */
    final private static String genByJuliakPkgName = "juliak.generated";
    
    static private Loader loader;
    
    protected Loader getLoader() {
        if( loader == null ) {
            loader = Juliak.get().getFcLoader();
        }
        return loader;
    }
    protected ClassLoader getClassLoader() {
        Object o = Platform.get().getLoader();
        return (ClassLoader) o;
    }
}
