/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.membrane;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.julia.control.content.SuperControllerNotifier;
import org.objectweb.fractal.juliak.control.binding.BindingControllerDef;
import org.objectweb.fractal.juliak.control.component.ComponentControllerDef;
import org.objectweb.fractal.juliak.control.content.SuperControllerDef;
import org.objectweb.fractal.juliak.control.name.NameControllerDef;
import org.objectweb.fractal.juliak.factory.InterfaceInstanceGenerator;


/**
 * Implementation of the control membrane for mPrimitive components.
 * mPrimitive components are primitive control components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class MPrimitiveImpl
    implements
        Component, BindingController, NameController, SuperControllerNotifier {
    
    private Type type;
    private Object content;
    
    public MPrimitiveImpl( Type type, Object content )
    throws InstantiationException {
        
        this.type = type;
        this.content = content;
                
        InterfaceInstanceGenerator itfgen = InterfaceInstanceGenerator.get();
        
        ComponentType ct = (ComponentType) type;
        InterfaceType[] its = ct.getFcInterfaceTypes();
        for (int i = 0; i < its.length; i++) {
            String name = its[i].getFcItfName();
            if( its[i].isFcCollectionItf() ) {
                // Julia naming convention for collection fcinterfaces
                name = "/collection/"+name;
            }
            Object itf = itfgen.generate(its[i],this,content,false);
            fcInterfaces.put(name,itf);
        }
        
        fcInterfaces.put(
                ComponentControllerDef.NAME,
                itfgen.generate(ComponentControllerDef.TYPE,this,this,false) );
        fcInterfaces.put(
                BindingControllerDef.NAME,
                itfgen.generate(BindingControllerDef.TYPE,this,this,false) );
        fcInterfaces.put(
                NameControllerDef.NAME,
                itfgen.generate(NameControllerDef.TYPE,this,this,false) );
        fcInterfaces.put(
                SuperControllerDef.NAME,
                itfgen.generate(SuperControllerDef.TYPE,this,this,false) );
    }
    
    
    // -----------------------------------------------------------------
    // Implementation of the Component interface
    // -----------------------------------------------------------------
    
    public Type getFcType() {
        return type;
    }

    public Object getFcInterface(String interfaceName)
    throws NoSuchInterfaceException {

        if( fcInterfaces.containsKey(interfaceName) ) {
            Object itf = fcInterfaces.get(interfaceName);
            return itf;
        }

        // TODO support dynamically created collection interface instances
        
        if( interfaceName.equals("/content") ) {
            return content;
        }
        
        throw new NoSuchInterfaceException(interfaceName);
    }
    
    public Object[] getFcInterfaces() {        
        return fcInterfaces.values().toArray();
    }
    
    protected Map fcInterfaces = new HashMap();
    
    
    // -----------------------------------------------------------------
    // Implementation of the BindingController interface
    // -----------------------------------------------------------------
    
    public String[] listFc() {
        Set names = new HashSet();
        for (Iterator iter = fcInterfaces.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            ComponentInterface itf = (ComponentInterface) entry.getValue();
            Object target = itf.getFcItfImpl();
            if( target != null && (target instanceof ComponentInterface) ) {
                String itfName = (String) entry.getKey();
                names.add(itfName);
            }
        }
        return (String[]) names.toArray( new String[names.size()] );
    }
    
    public Object lookupFc(String arg0) throws NoSuchInterfaceException {
        
        // Singleton client interfaces
        if( fcInterfaces.containsKey(arg0) ) {
            Object o = fcInterfaces.get(arg0);
            ComponentInterface itf = (ComponentInterface) o;
            InterfaceType it = (InterfaceType) itf.getFcItfType();
            if( it.isFcClientItf() ) {
                return itf.getFcItfImpl();
            }
        }
        
        // Collection client interfaces
        if( fcInterfaces.containsKey("/collection/"+arg0) ) {
            List bound = new ArrayList();
            for (Iterator iter = fcInterfaces.keySet().iterator(); iter.hasNext();) {
                String itfName = (String) iter.next();
                if( itfName.startsWith(arg0) ) {
                    Object o = lookupFc(itfName);
                    bound.add(o);
                }
            }
            return bound;
        }
        
        throw new NoSuchInterfaceException(arg0);
    }
    
    public void bindFc(String arg0, Object arg1)
    throws NoSuchInterfaceException, IllegalBindingException,
    IllegalLifeCycleException {
        
        /*
         * Let the client interface delegate the call to the target.
         * Register the binding.
         */
        Object o = getFcInterface(arg0);
        ComponentInterface itf = (ComponentInterface) o;        
        itf.setFcItfImpl(arg1);
    }
    
    public void unbindFc(String arg0)
    throws NoSuchInterfaceException, IllegalBindingException,
    IllegalLifeCycleException {

        Object o = getFcInterface(arg0);
        ComponentInterface itf = (ComponentInterface) o;
        itf.setFcItfImpl(null);
    }
    
    
    // -----------------------------------------------------------------
    // Implementation of the NameController interface
    // -----------------------------------------------------------------
    
    private String fcName;
    
    public String getFcName() {
        return fcName;
    }

    public void setFcName(String arg0) {
        fcName = arg0;
    }
    
    
    // -----------------------------------------------------------------
    // Implementation of the SuperControllerNotifier interface
    // -----------------------------------------------------------------
    
    protected List fcSupers = new ArrayList(1);
    
    public Component[] getFcSuperComponents() {
        return (Component[]) fcSupers.toArray(new Component[fcSupers.size()]);
    }

    public void addedToFc( Component c ) {
        fcSupers.add(c);
    }
    
    public void removedFromFc( Component c ) {
        fcSupers.remove(c);
    }
    
    
    // -----------------------------------------------------------------
    // Implementation specific
    // -----------------------------------------------------------------
    
    /**
     * Set the content instance associated to this control component.
     * 
     * This method is used in the membrane factory when instanciating an
     * interceptor component.
     */
    public void setFcContent( Object content ) {
        this.content = content;
    }
}
