/***
 * Julia
 * Copyright (C) 2005-2007 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.conform;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.Interceptor;
import org.objectweb.fractal.julia.conform.Test;
import org.objectweb.fractal.julia.conform.components.C;
import org.objectweb.fractal.julia.conform.components.I;
import org.objectweb.fractal.julia.conform.controllers.BasicStatController;
import org.objectweb.fractal.juliak.control.interceptor.InterceptorController;
import org.objectweb.fractal.util.Fractal;

/**
 * JUnit test for the interceptor controller of Juliak.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class TestInterceptorController extends Test {

    protected Component boot;
    protected TypeFactory tf;
    protected GenericFactory gf;

    protected ComponentType t;
    protected Component c;

    // -------------------------------------------------------------------------
    // Constructor and setup
    // -------------------------------------------------------------------------

    public TestInterceptorController(final String name) {
      super(name);
    }

    protected void setUp () throws InstantiationException, NoSuchInterfaceException {
      boot = Fractal.getBootstrapComponent();
      tf = Fractal.getTypeFactory(boot);
      gf = Fractal.getGenericFactory(boot);
      t = tf.createFcType(new InterfaceType[] {
              tf.createFcItfType("server", I.class.getName(), false, false, false),
              tf.createFcItfType("servers", I.class.getName(), false, false, true),
              tf.createFcItfType("client", I.class.getName(), true, true, false),
              tf.createFcItfType("clients", I.class.getName(), true, true, true)
            });
      setUpComponents();
    }

    protected void setUpComponents () throws InstantiationException {
        c = gf.newFcInstance(t, "primitive", C.class.getName());
    }
    
    // -------------------------------------------------------------------------
    // Test InterceptorController
    // -------------------------------------------------------------------------

    public void testInterceptorController() throws NoSuchInterfaceException {
        InterceptorController ic = (InterceptorController)
            c.getFcInterface(InterceptorController.NAME);
        assertNotNull(ic);
    }

    // -------------------------------------------------------------------------
    // Test getFcInterceptors
    // -------------------------------------------------------------------------

    public void testGetInterceptors() throws NoSuchInterfaceException {
        checkInterceptor("server");
        checkNoInterceptor("client");
    }

    public void testCollectionGetInterceptors() throws NoSuchInterfaceException {
        checkInterceptor("servers");
        checkInterceptor("servers0");
        checkNoInterceptor("clients");
        checkNoInterceptor("clients0");
    }
    
    protected void checkInterceptor(String itfName) throws NoSuchInterfaceException {
        InterceptorController ic = (InterceptorController)
            c.getFcInterface(InterceptorController.NAME);
    
        Object itf = c.getFcInterface(itfName);
        if( ! (itf instanceof ComponentInterface) ) {
            fail("Interceptor should implement ComponentInterface");
        }
        ComponentInterface citf = (ComponentInterface) itf;
        
        Interceptor[] interceptors = ic.getFcInterceptors(citf);
        assertEquals(1,interceptors.length);
        
        Interceptor i0 = interceptors[0];
        if( !(i0 instanceof I) ) {
            fail("Interceptor should implement I");
        }
    }

    protected void checkNoInterceptor(String itfName) throws NoSuchInterfaceException {
        InterceptorController ic = (InterceptorController)
            c.getFcInterface(InterceptorController.NAME);
    
        Object itf = c.getFcInterface(itfName);
        if( ! (itf instanceof ComponentInterface) ) {
            fail("Interceptor should implement ComponentInterface");
        }
        ComponentInterface citf = (ComponentInterface) itf;
        
        Interceptor[] interceptors = ic.getFcInterceptors(citf);
        assertEquals(0,interceptors.length);
    }

    // -------------------------------------------------------------------------
    // Test addFcInterceptors and removeFcInterceptors
    // -------------------------------------------------------------------------

    public void testAddRemoveInterceptor() throws NoSuchInterfaceException, IllegalLifeCycleException {
        checkAddRemoveInterceptor("server");
    }
    
    public void testCollectionAddRemoveInterceptor() throws NoSuchInterfaceException, IllegalLifeCycleException {
        checkAddRemoveInterceptor("servers");
        checkAddRemoveInterceptor("servers0");
    }
    
    protected void checkAddRemoveInterceptor(String itfName) throws NoSuchInterfaceException, IllegalLifeCycleException {
        
        LifeCycleController lc = Fractal.getLifeCycleController(c);
        lc.startFc();
        
        InterceptorController ic = (InterceptorController)
            c.getFcInterface(InterceptorController.NAME);

        Object itf = c.getFcInterface(itfName);
        ComponentInterface citf = (ComponentInterface) itf;
        I iserver = (I) itf;
        
        // Add the interceptor
        IInterceptor ii = new IInterceptor();
        ic.addFcInterceptor(citf, ii);
        Interceptor[] interceptors = ic.getFcInterceptors(citf);
        assertEquals(2,interceptors.length);
        assertEquals(0, ii.fcCounter);
        
        // Call m(), check that the interceptor has been executed
        iserver.m(true);
        assertEquals(1, ii.fcCounter);
        
        // Remove the interceptor
        ic.removeFcInterceptor(citf, ii);
        interceptors = ic.getFcInterceptors(citf);
        assertEquals(1,interceptors.length);
        
        // Call m(), check that the interceptor has not been executed
        iserver.m(true);
        assertEquals(1, ii.fcCounter);
    }
    
    private static class IInterceptor extends BasicStatController implements Interceptor, I {

        public Object getFcItfDelegate() {
            return delegate;
        }

        public void setFcItfDelegate(Object delegate) {
            this.delegate = delegate;
        }
        
        private Object delegate;

        public Object clone () {
            return null;
        }
        
        public void initFcController(InitializationContext ic) throws InstantiationException {
        }

        public void m(boolean v) {
            fcCounter++;
        }

        public void m(byte v) {
        }

        public void m(char v) {
        }

        public void m(short v) {
        }

        public void m(int v) {
        }

        public void m(long v) {
        }

        public void m(float v) {
        }

        public void m(double v) {
        }

        public void m(String v) {
        }

        public void m(String[] v) {
        }

        public boolean n(boolean v, String[] w) {
            return false;
        }

        public byte n(byte v, String w) {
            return 0;
        }

        public char n(char v, double w) {
            return 0;
        }

        public short n(short v, float w) {
            return 0;
        }

        public int n(int v, long w) {
            return 0;
        }

        public long n(long v, int w) {
            return 0;
        }

        public float n(float v, short w) {
            return 0;
        }

        public double n(double v, char w) {
            return 0;
        }

        public String n(String v, byte w) {
            return null;
        }

        public String[] n(String[] v, boolean w) {
            return null;
        }
        
    }
}
